%%%%% ps3 question 3: precautionary saving by backwards induction

clearvars;
close all;
clc;

%%%%% economic parameters

alpha = 1;          %% CRRA (=1/IES)
beta  = 0.95;       %% time discount factor   
R     = 1/beta;     %% asset return

%%%%% income distribution

mu    = -0.125;
sigma = 0.50;

%%%%% horizon

T     = 75; 

%%%%% numerical parameters

max_iter = 500;      %% maximum number of iterations
tol      = 1e-7;     %% treat numbers smaller than this as zero

%%%%% grid for income shocks

ny       = 33;       %% number of nodes for quadrature (shocks)

% quadrature nodes and weights
[ey, wy] = qnwunif(ny,1e-9,1-1e-9);
ey       = mu+sigma*norminv(ey,0,1); 
ygrid    = exp(ey);
ymin     = min(ygrid);
ymax     = max(ygrid);

%%%%% grid for cash on hand

nx       = 33;       %% number of breakpoints for x grid
curv     = 0.25;     %% (curv = 0 log-spaced, curv = 1 linear)

xmin     = 1e-1;
xmax     = 2*ymax/(1-beta);
xgrid    = nodeunif(nx, xmin.^curv, xmax.^curv).^(1/curv);

%%%%% put in a structure to pass to other functions

parameters.alpha = alpha;
parameters.beta  = beta;
parameters.mu    = mu;
parameters.sigma = sigma;
parameters.T     = T;

parameters.ygrid = ygrid;
parameters.wy    = wy;

%%%%% setup state space using CompEcon tools

fspace   = fundef({'spli', xgrid}); % function space structure

xgrid    = funnode(fspace);% nodes where we solve the problem
Phi      = funbas(fspace); % matrix of collocation basis vectors 
                           % Phi_{ij} = phi_j(k_i) 

xmin     = xgrid(1);
xmax     = xgrid(end); 

%%%%% bounds on consumption

cmin     = tol;
cmax     = xmax;

%%%%% form collection of states

s        = xgrid;

%%%%%

if alpha==1
    
    u = @(c) log(c);
    
else 
    
    u = @(c) (c.^(1-alpha)-1)./(1-alpha);
    
end

Vt = zeros(numel(xgrid),T+1);
Ct = zeros(numel(xgrid),T+1);

vx = u(xgrid);


Vt(:,1) = vx;
Ct(:,1) = xgrid;

for t=2:T+1

fprintf(' age remaining %4i \n',t-1);    
    
a   = Phi\Vt(:,t-1);

cx  = solve_brent('rhs_bellman',s,parameters,a,fspace,cmin,cmax,tol);

vx  = rhs_bellman(cx,s,parameters,a,fspace);

Ct(:,t) = cx;
Vt(:,t) = vx;

end

%selection = [T+1,26,11,2,1]; %% periods remaining
selection = [T+1,51,26,11,2,1]; %% periods remaining
%selection = [T+1,76,51,26,11,2,1]; %% periods remaining

N         = numel(selection);

P         = brewermap(7,'*BuPu');

figure(1)
set(gcf,'DefaultLineLineWidth', 2);
axes('ColorOrder',P,'NextPlot','replacechildren')
plot(xgrid,Vt(:,selection),'LineWidth',2)
set(get(gcf,'CurrentAxes'),'FontSize',16,'LineWidth', 1.5)
ylabel('value function $v_t(x)$','Interpreter','latex')
xlabel('cash-on-hand, $x$','Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
t = title('$v_t(x)$');
set(t,'Interpreter','latex');
l = legend('$t=0$','$t=1$','$t=10$','$t=25$','$t=50$','$t=75$','Location','Best');
set(l,'Interpreter','latex');

figure(2)
set(gcf,'DefaultLineLineWidth', 2);
axes('ColorOrder',P,'NextPlot','replacechildren')
plot(xgrid,Ct(:,selection),'LineWidth',2)
set(get(gcf,'CurrentAxes'),'FontSize',16,'LineWidth', 1.5)
ylabel('consumption function $c_t(x)$','Interpreter','latex')
xlabel('cash-on-hand, $x$','Interpreter','latex')
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')
t = title('$c_t(x)$');
set(t,'Interpreter','latex');
l = legend('$t=0$','$t=1$','$t=10$','$t=25$','$t=50$','$t=75$','Location','Best');
set(l,'Interpreter','latex');




